/* OGLHelper.c -- helper process for magic with OpenGL which signals the
 *		  parent (magic) whenever there is input available.
 */

#include <stdio.h>
#include <signal.h>
#include <GL/glut.h>

/* 
 * Portability stuff
 */


/* Some machines have signal handlers returning an int, while other machines
 * have it returning a void.  If you have a machine that requires ints put 
 * it in the list of machines in misc/magic.h.
 */
#ifdef	SIG_RETURNS_INT
#define	sigRetVal	int
#else
#define	sigRetVal	void
#endif


/*
 * Main program
 */

int readPipe,writePipe;	/* pipe file descriptor to magic process */
int parentID;		/* process id of parent */
Display *grOGLdpy;

sigRetVal MapWindow();

main (argc, argv)
    int argc;
    char **argv;
{
    XEvent xevent;

    if (argc <= 1) {
	fprintf(stderr, "%s: expecting two file-descriptor numbers.\n\
	    \t(This program should be run only by magic itself.)\n", argv[0]);
	exit(1);
    }

    sscanf(argv[1], "%d %d", &readPipe,&writePipe);
    grOGLdpy = XOpenDisplay(0);
    if (grOGLdpy == (Display *) 0) {
	fprintf(stderr,"%s: XOpenDisplay failed.\n", argv[0]);
	exit(1);
    }

    parentID = getppid();

    signal(SIGINT,SIG_IGN);
    signal(SIGQUIT,SIG_IGN);
    signal(SIGTERM,MapWindow); 
#ifdef SIGTSTP
    signal(SIGTSTP,SIG_IGN);
#endif
#ifdef SIGCONT
    signal(SIGCONT,SIG_IGN);
#endif

    while (1)
    {
	XNextEvent(grOGLdpy, &xevent);
	if (isString(xevent) ==0) 
	{
	    write(writePipe, &xevent, sizeof(XEvent));
	    kill(parentID, SIGIO);
	}
    }
}

isString (event)
    XEvent event;
{
    if (event.type == KeyPress) 
    {
	XKeyPressedEvent *KeyPressedEvent = (XKeyPressedEvent *) &event;
	char inChar[10],c,*p;
	int nbytes;
	KeySym keysym;

	nbytes = XLookupString(KeyPressedEvent,inChar,sizeof(inChar),
			 &keysym,NULL);
	p = inChar;
	while (nbytes--)
	{
	     if ( (c = *p++) == 3)
	     {
	     	  kill(parentID, SIGINT);
	     } 
	     else
	     {
	          write(writePipe, &event, sizeof(XEvent));
	          write(writePipe, &c, sizeof(char));
	          kill(parentID, SIGIO);
	     }
	} 
	return 1;
    }
    else return 0;
}

sigRetVal
MapWindow()
{
    Window window;

    if ( read(readPipe, (char *)&window, sizeof(Window)) == sizeof(Window))
    {
	XSelectInput(grOGLdpy, window,
		     KeyPressMask|ButtonPressMask|
		     ButtonReleaseMask|ExposureMask|
		     StructureNotifyMask|
		     OwnerGrabButtonMask);
	XSync(grOGLdpy,1);

#ifdef SYSV
    signal(SIGTERM, MapWindow);
#endif SYSV

    }
    else
	fprintf(stderr,"xhelper: read on pipe failed\n");
}
